@@ -49,6 +49,19 @@ def Theta(tempk,pres,pref=100000.):
 
     return tempk*(pref/pres)**(Rs_da/Cp_da)
 
+def TempK(theta,pres,pref=100000.):
+    """Inverts Theta function."""
+
+    try:
+	minpres=min(pres)
+    except TypeError:
+	minpres=pres
+
+    if minpres<2000:
+	print "WARNING: P<2000 Pa; did you input a value in hPa?"
+
+    return theta*(pres/pref)**(Rs_da/Cp_da)
+
 def ThetaE():
     """Equivalent potential temperature"""
     raise NotImplementedError
